#!/usr/bin/env python
'''
copyright 2010 jezra lickter http://www.jezra.net

heybuddy is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

heybuddy is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with heybuddy.	If not, see <http://www.gnu.org/licenses/>.
'''
import gtk,gobject
import sys, os
from xml.dom.minidom import parse, parseString
import re
import time
#this might be on Maemo
from PlatformSpecific import has_hildon,links_unavailable
#what are the custom classes that I need?
from Communicator import Communicator
from Configuration import Configuration
from XMLProcessor import XMLProcessor
from ImageCache import ImageCache
from StatusIcon import StatusIcon
from GroupPage import GroupPage
from About import About
from SettingsPage import SettingsPage
import MainWindow
from FilterPage import *
from DentScroller import DentScroller
from UserPage import UserPage
from ScrollPage import ScrollPage
from Dent import Dent
from ContextPage import ContextPage
#for notifications
from Notify import Notify, has_pynotify
#this might be on Maemo
try:
	import hildon
	has_hildon = True
except:
	has_hildon = False
	
global app_name
global version
global branch
app_name = "heybuddy"
version = "0.2.0"
branch = "testing"

#make a fake enumeration
DENT,MENTION,DIRECT,CONTEXT,USER,GROUP,FILTER,ACCOUNT,ABOUT = range(9)
highest_id={DENT:0,MENTION:0,DIRECT:0}
#create a fake enum for options
class application :
	def __init__(self):
		#init threading
		gobject.threads_init()
		#keep track of the window size
		self.window_height=0
		self.window_width=0
		self.window_is_hidden = False
		self.has_status_icon = False
		#keep track of the initial conversation id so that conversations don't colide
		self.conversation_id="0"
		#keep track of the filters
		self.filters ={'tags':[],'users':[],'groups':[]}
		self.options={'run_as_tray_app':False,'context_backwards':False,'no_avatars':False,'notifications':True,'notify_replies':False}
		#keep track of the direct_message requests
		self.last_direct_message_time=0
		#something something direct message
		self.is_direct_message_mode=False
		self.direct_message_to=None
		self.waiting_requests=0
		self.is_first_dents = True
		#keep track of the respond to id
		self.respond_to_id=0
		self.pre_group_page=None
		self.pre_user_page=None
		#what are the assets?
		asset_dir = "assets"
		heybuddy_dir = os.path.dirname( os.path.realpath( __file__ ) )
		self.readme_file	= os.path.join(heybuddy_dir,"README.txt")
		self.standard_icon_path = os.path.join(heybuddy_dir,asset_dir,"icon.png")
		self.direct_icon_path = os.path.join(heybuddy_dir,asset_dir,"direct_icon.png")
		self.networking_icon_path = os.path.join(heybuddy_dir,asset_dir,"icon1.png")
		self.throbber_icon_path = os.path.join(heybuddy_dir,asset_dir,"throbber.gif")
		self.default_icon_path = self.standard_icon_path
		self.conf = Configuration(app_name)
		self.xmlprocessor = XMLProcessor()
		#create a regex to replace parts of a dent
		self.regex_tag = re.compile("(^| )(#[A-z0-9_-]+)")
		self.regex_group = re.compile("(^| )(![A-z0-9_-]+)")
		self.regex_user=re.compile("(^| )(@[A-z0-9_]+)")
		self.regex_url=re.compile("(http[s]?://.*?)(\s|$)",re.IGNORECASE)
		#create filter regex
		self.regex_filter_tags = re.compile("#([A-z0-9_-]+)")
		self.regex_filter_groups = re.compile("!([A-z0-9_-]+)")
		#get the initial dents
		self.initial_dents = self.conf.get('settings','initial_dents',default='20' )
		#build the gui
		self.build_gui()
		
		#create the communicator
		self.comm = Communicator(app_name)
		#set the initial dents
		self.comm.set_initial_dents( self.initial_dents )
		
		self.comm.connect('statusesXML',self.process_statusesXML,self.dentspage)
		self.comm.connect('mentionsXML',self.process_statusesXML,self.mentionspage)
		self.comm.connect('group-statusesXML',self.process_statusesXML,self.grouppage)
		self.comm.connect('user-statusesXML',self.process_statusesXML,self.userpage)
		self.comm.connect('direct_messagesXML',self.process_statusesXML,self.directspage,True)
		self.comm.connect('conversationXML',self.process_conversationXML)
		self.comm.connect('userXML',self.process_userXML)
		self.comm.connect('groupXML',self.process_groupXML)
		self.comm.connect('new-statusXML',self.process_new_statusXML)
		self.comm.connect('user_is_friendXML',self.process_user_is_friendXML)
		self.comm.connect('user_is_memberXML',self.process_user_is_memberXML)
		self.comm.connect('friendshipXML',self.process_friendshipXML)
		self.comm.connect('exception-caught',self.communication_exception)
		self.comm.connect('widget-image',self.process_widget_image)
		self.comm.connect('direct-messageXML',self.simple_decrement)
		#create an image cacher thingy
		self.imagecache = ImageCache()
		self.imagecache.set_disabled( self.conf.get_bool_option("no_avatars") )
		self.imagecache.connect('get-widget-image', self.get_widget_image)
	
		#Create notify-er if has pynotify
		if has_pynotify:
			self.notify = Notify()

		#align the window
		self.align_window()
		# do we need to create the status icon
		if self.conf.get_bool_option('run_as_tray_app'):
			self.create_status_icon()
		
	def build_gui(self):
		#build the GUI 
		self.mainwindow=MainWindow.MainWindow(app_name,version,branch)
		try:
			self.mainwindow.set_icon_from_file(self.default_icon_path)
		except:
			print "failed to load %s" % (icon_path)
		self.mainwindow.connect("quit", self.quit_triggered )
		self.mainwindow.connect("window-to-tray", self.window_to_tray )
		self.mainwindow.connect("update-status",self.update_status)
		self.mainwindow.connect("get-conversation",self.get_conversation)
		self.mainwindow.connect("show-user",self.get_user_info)
		self.mainwindow.connect("show-group",self.get_group_info)
		self.mainwindow.connect("notebook-page-change",self.page_change)
		self.mainwindow.connect('follow-user',self.follow_user)
		self.mainwindow.connect('join-group',self.join_group)
		self.mainwindow.connect('clear-respond-to-id',self.clear_respond_to_id)
		#keep track of the window size
		self.mainwindow.connect('size-allocate', self.window_size_allocate)
		#self.mainwindow.connect('window-state-event', self.mainwindow_state_event)
		self.mainwindow.connect('delete-event', self.mainwindow_delete_event)
		
		# create and add all of the pages
		self.dentspage = ScrollPage()
		self.mainwindow.notebook.append_page(self.dentspage,gtk.Label("Dents") )
		
		self.mentionspage = ScrollPage()
		self.mainwindow.notebook.append_page(self.mentionspage,gtk.Label("Mentions") )
		
		self.directspage = ScrollPage()
		self.mainwindow.notebook.append_page(self.directspage,gtk.Label("Directs") )
		
		#make the conversation page
		self.contextpage=ContextPage()
		self.contextpage.connect('context-page-hide',self.hide_contextpage)
		#add the contextpage
		self.mainwindow.notebook.append_page(self.contextpage,gtk.Label("Context") )
		
		#create a user page
		self.userpage=UserPage()
		self.userpage.connect('user-page-hide',self.hide_userpage)
		self.userpage.connect('follow-user',self.follow_user)
		self.userpage.connect('direct-clicked',self.direct_clicked)
		##COmmented out 9.4.10 x1101 
		#self.userpage.connect('block-create',self.block_create)
		#self.userpage.connect('block-destroy',self.block_destroy)
		
		#add the userpage
		self.mainwindow.notebook.append_page(self.userpage,gtk.Label("User") )
		
		#create a Group page
		self.grouppage=GroupPage()
		self.grouppage.connect('group-page-hide',self.hide_grouppage)
		self.grouppage.connect('join-group',self.join_group)
		#add the Grouppage
		self.mainwindow.notebook.append_page(self.grouppage,gtk.Label("Group") )
		
		#create the filters page
		self.filterpage = FilterPage()
		#add the groupage to the notebook
		self.mainwindow.notebook.append_page(self.filterpage,gtk.Label("Filters") )
		#connect the filter page signals
		self.filterpage.connect('add-tag-filter', self.add_tag_filter )
		self.filterpage.connect('remove-tag-filter', self.remove_tag_filter )
		self.filterpage.connect('add-group-filter', self.add_group_filter )
		self.filterpage.connect('remove-group-filter', self.remove_group_filter )
		self.filterpage.connect('add-user-filter', self.add_user_filter )
		self.filterpage.connect('remove-user-filter', self.remove_user_filter )
		
		#create and add the account page
		self.settingspage = SettingsPage(has_hildon,has_pynotify)
		self.settingspage.set_initial_dents( self.initial_dents )
		#update the configuration of the settings
		self.settingspage.connect("update-account",self.update_account)
		self.options['run_as_tray_app'] = self.conf.get_bool_option('run_as_tray_app')
		self.settingspage.set_run_as_tray_app( self.options['run_as_tray_app']	)
		self.options['context_backwards']	= self.conf.get_bool_option('context_backwards')
		self.settingspage.set_context_backwards( self.options['context_backwards'] )
		self.options['no_avatar']= self.conf.get_bool_option('no_avatars')
		self.settingspage.set_no_avatars( self.options['no_avatar'] )
		self.options['notifications']= self.conf.get_bool_option('notifications')
		self.settingspage.set_notifications( self.options['notifications'] )
		self.options['notify_replies']= self.conf.get_bool_option('notify_replies')
		self.settingspage.set_notify_replies( self.options['notify_replies'] )
		link_color = self.conf.get('settings','link_color',default='blue')
		self.settingspage.set_link_color( link_color )
		self.theme_link_color(link_color)
		#connect the setting signals
		self.settingspage.connect('option-run-as-tray-app', self.option_changed, 'run_as_tray_app')
		self.settingspage.connect('option-context-backwards',self.option_changed, 'context_backwards')
		self.settingspage.connect('option-no-avatars',self.option_changed, 'no_avatars')
		self.settingspage.connect('option-notifications',self.option_changed, 'notifications')
		self.settingspage.connect('option-notify-replies',self.option_changed, 'notify_replies')
		self.settingspage.connect('initial-dents',self.set_initial_dents)
		self.settingspage.connect('link-color',self.set_link_color)
		#add the settings to the mainwindow
		self.mainwindow.add_notebook_page(self.settingspage,gtk.Label("Settings"))
		#create and add the about page
		about = About(version,branch,self.standard_icon_path,self.readme_file)
		self.mainwindow.add_notebook_page( about, gtk.Label("About") )
		self.mainwindow.show_some()
		#hide some stuff
		self.grouppage.hide_all()
		self.userpage.hide_all()
		self.contextpage.hide_all()
		
	def align_window(self):
		#try to set the mainwindows size based on conf info
		try:
			w = int( self.conf.get('window_info','width') )
			h = int( self.conf.get('window_info','height') )
			self.mainwindow.resize(w,h)
		except:
			pass
		try:
			x = int( self.conf.get('window_info','x') )
			y = int( self.conf.get('window_info','y') )
			self.mainwindow.move(x,y)
		except:
			pass
		
		#get the filters
		try: 
			#read the filters
			filters = self.conf.get('settings','filters',True)
			#the filters should be an array
			if len(filters):
				self.filters = filters
				#set the filters
				self.filterpage.set_tag_filters( self.filters['tags'] )
				self.filterpage.set_user_filters( self.filters['users'] )
				self.filterpage.set_group_filters( self.filters['groups'] )
		except:
			pass
	
	def create_status_icon(self):
		if not self.has_status_icon:
			#create the statusicon
			self.statusicon = StatusIcon()
			self.statusicon.connect('activate', self.status_clicked )
			self.statusicon.connect('quit-selected',self.quit_app)
			self.statusicon.set_icon( self.standard_icon_path )
		self.statusicon.set_property("visible",True)
		self.has_status_icon=True
	
	def destroy_status_icon(self):
		self.statusicon.set_property("visible",False)
		self.has_status_icon=False
		
	def update_account(self,widget,n,p):
		if n!=self.conf.name() or p!=self.conf.password():
			#TODO:check if these are valid
			#self.comm.check_validity(n,p)
			self.valid_account_info(n,p)
			self.mainwindow.message_label.hide()
			
	def increment_requests(self):
		self.waiting_requests+=1
		if self.waiting_requests==1:
			try:
				self.mainwindow.set_icon_from_file(self.networking_icon_path)
			except:
				print "failed to load %s" % (self.networking_icon_path)
			try:
				self.mainwindow.set_throbber(self.throbber_icon_path)
			except:
				print "failed to load %s" % (self.throbber_icon_path)
			if self.has_status_icon:
				self.statusicon.set_icon( self.networking_icon_path)
				
	def decrement_requests(self):
		self.waiting_requests-=1
		if self.waiting_requests==0:
			try:
				self.mainwindow.set_icon_from_file(self.default_icon_path)
			except:
				print "failed to load %s" % (self.default_icon_path)
			try:
				self.mainwindow.set_throbber(self.default_icon_path)
			except:
				print "failed to load %s" % (self.default_icon_path)
			if self.has_status_icon:
				self.statusicon.set_icon( self.default_icon_path )
		if self.waiting_requests<0:
			self.waiting_requests=0
	
	def valid_account_info(self,n,p):
		self.conf.name( n )
		self.conf.password( p )
		self.conf.save()
		#update the comm
		self.comm.set_name_and_password( n,p )
		#switch to the dents page
		self.mainwindow.set_notebook_page(DENT) 
		self.get_statuses()

	def get_statuses(self):
		self.increment_requests()
		self.comm.get_statuses(highest_id[DENT])
	
	def get_mentions(self):
		self.increment_requests()
		self.comm.get_mentions(highest_id[MENTION])	
	
	def get_direct_messages(self):
		now = time.time()
		#has it been 3 minutes since the last request?
		if now-(3*60) > self.last_direct_message_time:
			self.increment_requests()
			self.comm.get_direct_messages(highest_id[DIRECT])	
			self.last_direct_message_time=now
	
	def run(self):
		#start the communicator
		self.comm.start()
		self.main_loop = gobject.MainLoop()
		#add a function to the main loop
		gobject.timeout_add(90*1000, self.update_statuses )
		gobject.timeout_add(3*60*1000, self.update_mentions)
		#we have to temporarily show the various pages or nothing works
		''' why the fuck do I need to do this? '''
		self.mainwindow.set_notebook_page(MainWindow.DIRECT)
		self.mainwindow.set_notebook_page(MainWindow.MENTION)
		self.mainwindow.set_notebook_page(MainWindow.DENT)
		#did the conf read a name and password?
		if( self.conf.name() !="" and self.conf.password()!=""):
			self.comm.set_name_and_password( self.conf.name(),self.conf.password() )
			#fill the Account tab name/pw fields to show the user that they're logged in
			self.settingspage.set_name(self.conf.name())
			self.settingspage.set_password(self.conf.password())
			#add a timout to get dents
			gobject.timeout_add(500,self.get_statuses )
			gobject.timeout_add(600,self.get_mentions )
			#gobject.timeout_add(1000,self.get_direct_messages )
		else:
			#set the focus on the account page; the last page
			self.mainwindow.set_notebook_page(MainWindow.ACCOUNT)
		
		self.mainwindow.run()
		self.main_loop.run()

		
		#if we ever get here, we should quit

	def update_statuses(self):
		self.get_statuses()
		return True
	
	def update_mentions(self):
		self.get_mentions()
		return True
	
	def update_direct_messages(self):
		self.get_direct_messages()
		return True
	
	def update_status(self,widget,text):
		self.increment_requests()
		#remove newlines from text
		text=text.replace("\n","")
		#is this direct message mode?
		if self.is_direct_message_mode:
			self.comm.send_direct_message(text,self.direct_message_to)
			self.set_is_direct_message_mode(False)
		else:
			self.comm.send_update_status(text,self.respond_to_id)
		self.clear_respond_to_id()
		
	def quit_app(self,widget=None):
		#quit the communicator loop
		self.comm.quit()
		#quit the main loop
		self.main_loop.quit()
		
	def get_widget_image(self,imagecache,image):
		self.comm.get_widget_image(image)
	
	def get_conversation(self,id,conversation_id):
		self.increment_requests()
		self.comm.get_conversation(id,conversation_id)
	
	def get_user_info(self,name):
		self.increment_requests()
		self.comm.get_user_info(name)
		self.increment_requests()
		self.comm.get_user_statuses(name)
	
	def get_group_info(self,name):
		self.increment_requests()
		self.comm.get_group_info(name)
	
	def follow_user(self,widget,name,bool):
		if bool:
			self.comm.friendship_create(name)
		else:
			self.comm.friendship_destroy(name)
	
	def join_group(self,widget,name,bool):
		if bool:
			self.comm.group_join(name)
		else:
			self.comm.group_leave(name)
	
	def process_friendshipXML(self,widget,text):
		pass
	
	def page_change(self,widget,old_id,new_id):
		if new_id==MainWindow.DIRECT:
			self.get_direct_messages()
		else:
			if self.is_direct_message_mode:
				self.set_is_direct_message_mode(False)
	
	def process_userXML(self,comm,text):
		self.decrement_requests()
		data = self.xmlprocessor.get_user_info(text)
		if data['profile_image_url']!=None:
			self.imagecache.add_image_to_widget(data['profile_image_url'],self.userpage)
		self.userpage.set_user_data(data)
	
	def process_groupXML(self,comm,text):
		self.decrement_requests()
		data = self.xmlprocessor.get_group_info(text)
		if data['stream_logo']!=None:
			self.imagecache.add_image_to_widget(data['stream_logo'],self.grouppage)
		'''#is this user a member of this group
		self.increment_requests()
		self.comm.get_user_is_member(data['id'])'''
		self.increment_requests()
		self.comm.get_group_statuses(data['id'])
		self.grouppage.set_group_data(data)
	
	def process_user_is_friendXML(self,comm,text):
		self.decrement_requests()
		is_friend = self.xmlprocessor.get_user_is_friend(text)
		self.mainwindow.display_user_is_friend( is_friend )
	
	def process_user_is_memberXML(self,comm,text):
		self.decrement_requests()
		is_member = self.xmlprocessor.get_user_is_member(text)
		self.mainwindow.display_user_is_member( is_member )
	
	def process_new_statusXML(self,comm,text):
		#a update status was sent, get all new statuses
		#self.update_statuses()	
		self.decrement_requests()
		#there is one statusa and it is the DOM
		status = self.xmlprocessor.get_dom(text)
		data = self.xmlprocessor.get_dent_data(status)
		data['markup'] = self.markup_dent_text(data['text'])
		#add the dent
		dent = self.connect_dent(data,self.dentspage)
		self.imagecache.add_image_to_widget(data['profile_image_url'],dent)
	
	def process_conversationXML(self,object,text,conversation_id):
		self.decrement_requests()
		#is this the current conversation Id? if not, then do nothing
		#convert to int because there is a problem with string comparison
		'''is one of the strings unicode?'''
		if int(self.conversation_id)==int(conversation_id):
			#get the status
			status = self.xmlprocessor.get_dom(text)
			data = self.xmlprocessor.get_dent_data(status)
			data['markup']=self.markup_dent_text(data['text'])
			#since this is a conversation, we can get rid of the in_reply_to_screen_name
			data['in_reply_to_screen_name']=None
			#tell the mainWindow to add the dent
			dent = self.connect_dent(data,self.contextpage,True, self.options['context_backwards'] )
			self.imagecache.add_image_to_widget(data['profile_image_url'],dent)
			if data['in_reply_to_id']!=None:
				#recursively get in_reply_to_ids
				self.get_conversation(id=data['in_reply_to_id'],conversation_id=conversation_id)

	def connect_dent(self,data,target_page,is_conv=False,conv_backwards=False,is_direct_dent=False,prune=0):
		#make the dent
		dent = Dent(data,is_direct=is_direct_dent)
		dent.connect("group-clicked", self.view_group)
		dent.connect("reply-clicked", self.reply_clicked)
		dent.connect("direct-clicked", self.direct_clicked)
		dent.connect("view-conversation-clicked", self.view_conversation)
		dent.connect("user-clicked", self.view_user)
		dent.connect("text-label-clicked",self.dent_text_clicked)
		dent.connect("redent-clicked",self.redent_clicked)
		dent.connect("favorite-clicked",self.favorite_clicked)
		dent.connect("unfavorite-clicked",self.unfavorite_clicked)
		target_page.dentScroller.add_dent( dent, is_conv, conv_backwards )
		if prune>0:
			target_page.dentScroller.prune(prune)
		return dent
		 
	def reply_clicked(self,dent,name):
		self.respond_to_id=dent.id
		#set the text buffer
		self.mainwindow.update_textbuffer.set_text("@%s " % (name) )
		self.mainwindow.update_textview.grab_focus()
	
	def favorite_clicked(self,dent):
		id=dent.id
		self.comm.favorite(id)
	
	def unfavorite_clicked(self,dent):
		id=dent.id
		self.comm.unfavorite(id)
	
	def direct_clicked(self,widget,name):
		self.direct_message_to = name
		if type(widget).__name__=="Dent":
			self.respond_to_id=widget.id
		else:
			self.respond_to_id=None
		self.set_is_direct_message_mode(True,name)
		#we should be on the directs page
		self.mainwindow.set_notebook_page(DIRECT)
		#set the text buffer
		self.mainwindow.update_textbuffer.set_text("")
		self.mainwindow.update_textview.grab_focus()
	
	def view_conversation(self,dent,id):
		self.conversation_id=id
		self.pre_context_page=self.mainwindow.notebook_current_page
		self.contextpage.dentScroller.clear()
		self.contextpage.show_all()
		#get conversation starting with id
		self.get_conversation(id,id)
		self.mainwindow.notebook.set_current_page(CONTEXT)
		
	def view_user(self,widget,screen_name):
		self.get_user_info(screen_name)
		self.pre_user_page=self.mainwindow.notebook_current_page
		#make the user page checkbox insensitive
		self.userpage.disable()
		#show the page 
		self.userpage.show_all()
		#clear the userpage stuff
		self.userpage.clear()
		#change to the user page
		self.mainwindow.set_notebook_page(USER)
				
	def get_dent_time(self,text):
		print text
		
	def markup_dent_text(self,text):
		#process the text to markup
		#xmlencode the string
		text = self.xmlprocessor.xmlentities(text)
		#regex find the highlights
		markup = self.regex_tag.sub(r'\1<b>\2</b>',text)
		#regex find users
		if has_hildon or links_unavailable:
			markup = self.regex_group.sub(r'\1<b>\2</b>',markup)
			markup = self.regex_user.sub(r'\1<b>\2</b>',markup)
		else:
			#find urls
			markup = self.regex_url.sub(r'<a href="\1">\1</a>\2',markup)
			markup = self.regex_user.sub(r'\1<a href="\2">\2</a>',markup)
			markup = self.regex_group.sub(r'\1<a href="\2">\2</a>',markup)
		return markup
	
	def process_statusesXML(self,object,text,target_page,is_direct=False):
		#based on the target, what is the counting thingy?
		count_ref=None
		if target_page==self.dentspage:
			count_ref = DENT
		elif target_page==self.mentionspage:
			count_ref = MENTION
		elif target_page==self.directspage:
			count_ref = DIRECT
		self.decrement_requests()
		#get a list of the statuses
		statuses = self.xmlprocessor.get_statuses(text,is_direct)
		#reverse the statuses list
		statuses.reverse()
		dent_count=0
		reply_count=0
		#what is the highest id of the target index?	
		for status in statuses:
			filtered_status = False
			data = self.xmlprocessor.get_dent_data(status)

			if count_ref!=None:
				highest_id[count_ref]=data['status_id']

			if target_page==self.dentspage:
				#we should check for filtration
				filtered = self.check_filtered( data )
				filtered_status=filtered[0]
			if filtered_status:
				print "Filtered dent #%s matching %s" % (data['status_id'],filtered[1])
			else:
				#get the markup
				data['markup'] = self.markup_dent_text(data['text'])
				dent = self.connect_dent(data,target_page,is_direct_dent=is_direct,prune=self.initial_dents*3)
				#if not the userpage
				if target_page!=self.userpage:
					#get the image for this dent
					self.imagecache.add_image_to_widget(data['profile_image_url'],dent)
				#if the target_page = 0 and not first_dents and not is_conf
				if target_page==self.dentspage and not self.is_first_dents:
					if "@"+self.conf.name() in data['markup']:					
						dent=self.connect_dent(data, self.mentionspage,prune=self.initial_dents*3 )
						self.imagecache.add_image_to_widget(data['profile_image_url'],dent)
						reply_count+=1
						if self.options['notify_replies'] and has_pynotify:
							print "Notifying for new reply"
							self.notify.notify_reply(data['screen_name'], data['markup'] )
							print "Showing @reply notification"
					else:
						dent_count+=1
					
		self.is_first_dents=False
		#get the related images
		self.imagecache.get_images()
		if (dent_count!=0 or reply_count!=0) and has_pynotify:
			self.prepare_notifications(dent_count,reply_count)
	'''
		Take in the statuses, check if user wants notifications
		count the number of notifications, and call notify methods
	'''
	def prepare_notifications(self,dents,replies):
		#count = len(statuses)
		if self.options['notifications']:
			if not self.options['notify_replies']:
				print "Showing notification"
				self.notify.notify(dents,replies)
			else:
				if dents!=0:
					self.notify.notify_updates(dents)
					print "Showing notifications"
	
	def process_widget_image(self,comm,data,name):
		self.imagecache.widget_image_add(data,name)	
		
	def communication_exception(self,communicator,code):
		self.decrement_requests()
		if code=="401": 
			#bad authorization
			self.mainwindow.set_message("Invalid Authorization: Your name or password is incorrect",MainWindow.ERROR_MESSAGE)
			self.mainwindow.set_notebook_page(MainWindow.ACCOUNT)
		elif code == "404":
			print 'not found'
		elif code == "503":
			print 'service unavailable'
		else:
			print 'unknown error: '+code
	
	#functions to handle the filtration system
	def add_tag_filter(self,winder,tag):
		if not tag in self.filters['tags']:
			self.filters['tags'].append(tag)
			self.filters['tags'].sort()
			self.filterpage.set_tag_filters( self.filters['tags'] )
		
	def remove_tag_filter(self,winder,tag):
		if tag in self.filters['tags']:
			self.filters['tags'].remove(tag)
			self.filterpage.set_tag_filters( self.filters['tags'] )
	
	def add_group_filter(self,winder,group):
		if not group in self.filters['groups']:
			self.filters['groups'].append(group)
			self.filters['groups'].sort()
			self.filterpage.set_group_filters( self.filters['groups'] )
		
	def remove_group_filter(self,winder,group):
		if group in self.filters['groups']:
			self.filters['groups'].remove(group)
			self.filterpage.set_group_filters( self.filters['groups'] )
			
	def add_user_filter(self,winder,user):
		if not user in self.filters['users']:
			self.filters['users'].append(user)
			self.filters['users'].sort()
			self.filterpage.set_user_filters( self.filters['users'] )
		
	def remove_user_filter(self,winder,user):
		if user in self.filters['users']:
			self.filters['users'].remove(user)
			self.filterpage.set_user_filters( self.filters['users'] )
	
	def check_filtered(self,data):
		if len(self.filters['tags'])==0 and len(self.filters['users'])==0 and len(self.filters['groups'])==0:
			return (False, None)
		#check the tags
		text = data['text']
		#what are the tags in the dent?
		dent_tags = self.regex_filter_tags.findall(text)
		for tag in dent_tags:
			for ftag in self.filters['tags']:
				if tag.lower()==ftag.lower():
					return (True, "tag: %s" % tag )
				
		#what are the groups in the dent?
		dent_groups = self.regex_filter_groups.findall(text)
		for group in dent_groups:
			for fgroup in self.filters['groups']:
				if group.lower()==fgroup.lower():
					return (True, "group: %s" % group )
				
		#check the user
		for user in self.filters['users']:
			if data['screen_name'].lower()==user.lower():
				return (True, "user: %s" % data['screen_name'] )
		
		return (False, None)
	
	def status_clicked(self,widget):
		if self.window_is_hidden==True:
			self.window_is_hidden=False
			self.mainwindow.show()
			self.align_window()
		#bring the window forward 
		self.mainwindow.present()
						 
	def window_size_allocate(self,widget,size):
		self.window_width = size[2]
		self.window_height = size[3]
			
	def save_app_info(self):
		#save the filters
		self.conf.set('settings','filters', self.filters ,True)
		#save the window info
		self.conf.set('window_info','width', self.window_width )
		self.conf.set('window_info','height', self.window_height )
		try:
			#some themes don't pass root origin?
			origin = self.mainwindow.window.get_root_origin()
			self.window_x = origin[0]
			self.window_y = origin[1]
			self.conf.set('window_info','x', self.window_x )
			self.conf.set('window_info','y', self.window_y )
		except:
			pass
		self.conf.save()
		
	def mainwindow_delete_event(self,window,event=None):
		self.save_app_info()
		#are we in tray mode?
		if self.conf.get_bool_option('run_as_tray_app') or self.has_status_icon:
			#is the status icon embedded?
			if self.statusicon.is_embedded():
				self.window_is_hidden=True
				self.mainwindow.hide_on_delete()
				#do not propogate the signal
				return True
		else:
			#we need to quit
			self.quit_app()
			
	def quit_triggered(self,widget):
		self.save_app_info()
		self.quit_app()
	
	def option_changed(self,widget,value,option):
		#save the option change in the configuration
		self.conf.set('options',option,value)
		self.options[option]=value
		self.conf.save()
		#is this the run_tray_app:
		if option=='run_as_tray_app':
			if value:
				self.create_status_icon()
			else:
				self.destroy_status_icon()
		elif option=='no_avatars':
			self.imagecache.set_disabled(value)
		 
	def redent_clicked(self,dent,name,text):
		self.respond_to_id=dent.id
		formatted_text= u'\u267A @%s: %s' % (name,text)
		self.mainwindow.update_textbuffer.set_text(formatted_text )
		self.mainwindow.update_textview.grab_focus()
	 
	def view_group(self,widget,group_name):
		#where were we?
		self.pre_group_page=self.mainwindow.notebook_current_page
		#make some flags
		self.grouppage.joined_checkbutton.set_sensitive(False)
		#make the grouppage visible
		self.grouppage.show_all()
		#clear the grouppage
		self.grouppage.clear()
		#switch to the group page
		self.mainwindow.set_notebook_page(GROUP)
		#get the group info
		self.get_group_info(group_name)

	def hide_contextpage(self,widget):
		#end any conversation in progress
		self.conversation_id="0"
		self.mainwindow.set_notebook_page( self.pre_context_page )
		self.contextpage.hide()

	def hide_grouppage(self,widget):
		self.mainwindow.set_notebook_page( self.pre_group_page )
		self.grouppage.hide()

	def hide_userpage(self,widget):
		self.mainwindow.set_notebook_page( self.pre_user_page )
		self.userpage.hide()

	def dent_text_clicked(self,dent,text_label):
		self.mainwindow.last_clicked_label = text_label

	def clear_respond_to_id(self,widget=None):
		self.respond_to_id=0
		
	def set_initial_dents(self,widget,value):
		self.initial_dents=value
		self.conf.set('settings','initial_dents',value)
		
	def set_link_color(self,widget,string):
		self.conf.set('settings','link_color',string)
		self.theme_link_color(string)
		
	def theme_link_color(self,color):
		style='''
		style "label" {
			GtkLabel::link-color="%s"
		}
		widget_class "*GtkLabel" style "label"
		''' % (color)
		gtk.rc_parse_string(style)
		#fuck you GTK for not make this work, more than once!
	
	##Commented out 9.4.10 x1101	
	#functions for blocking/unblocking
	#def block_create(self,widget,user_id):
	#	self.comm.block_create(user_id)
		
	#def block_destroy(self,widget,user_id):
	#	self.comm.block_destroy(user_id)
		
		
	def set_is_direct_message_mode(self,mode,user=None):
		self.is_direct_message_mode=mode
		self.mainwindow.set_is_direct_message_mode(mode,user)
		if mode:
			self.default_icon_path=self.direct_icon_path
		else:
			self.default_icon_path=self.standard_icon_path
		#if we aren't waiting on any requests, swap the graphic
		if self.waiting_requests==0:
			try:
				self.mainwindow.set_icon_from_file(self.default_icon_path)
			except:
				print "failed to load %s" % (self.default_icon_path)
			try:
				self.mainwindow.set_throbber(self.default_icon_path)
			except:
				print "failed to load %s" % (self.default_icon_path)
			
	def simple_decrement(self,text=None,data=None):
		self.decrement_requests()

	'''
	function window_to_tray
	sets the run_as_tray_app to true and minimizes the app
	'''
	def window_to_tray(self,window):
		if self.options['run_as_tray_app']:
			self.mainwindow_delete_event(window)

if(__name__=="__main__"):
	a = application()
	a.run()
